@@ -1,6 +1,6 @@
-#' `basic_cache` makes a cache that does not expire old entries.
+#' `permanent_cache` constructs a cache that does not expire old entries.
 #' It should be used in situations where you know the number of
-#' things to remember is bounded.
+#' values to remember is bounded.
 #' @rdname lru_cache
 #' @export
 permanent_cache <- function() {
@@ -15,27 +15,29 @@ permanent_cache <- function() {
   function(key, value, action="cache", ifnotfound=NULL) {
     switch(action,
            exists=exists(key, cache),
-           cache=if(exists(key, cache)) {
-             hits <<- hits+1L
+           cache=if (exists(key, cache)) {
+             hits <<- hits + 1L
              cache[[key]]
            } else {
-             misses <<- misses+1
-             used <<- used+1L
+             misses <<- misses + 1L
+             used <<- used + 1L
              cache[[key]] <<- value
+             value
            },
-           get=if(exists(key, cache)) {
-             hits <<- hits+1L
+           get=if (exists(key, cache)) {
+             hits <<- hits + 1L
+             cache[[key]]
            } else {
-             misses <- misses+1L
+             misses <- misses + 1L
              ifnotfound
            },
            set={
-             if(!exists(key, cache)) {used <- used+1L}
+             if (!exists(key, cache)) used <<- used + 1L
              cache[[key]] <<- value
            },
-           rm=if(exists(key, cache)) {
-             used <<- used-1L
-             expired <<- expired+1L
+           rm=if (exists(key, cache)) {
+             used <<- used - 1L
+             expired <<- expired + 1L
              rm(list=key, envir=cache)
            })
   }
@@ -57,10 +59,10 @@ lru_cache <- function(size = 10000) {
   pred <- new.env(hash=TRUE, parent=emptyenv(), size=size)
   succ <- new.env(hash=TRUE, parent=emptyenv(), size=size)
 
-  hits <- 0
-  misses <- 0
-  expired <- 0
-  used <- 0
+  hits <- 0L
+  misses <- 0L
+  expired <- 0L
+  used <- 0L
 
   pred$TAIL <- "HEAD"
   succ$HEAD <- "TAIL"
@@ -68,7 +70,7 @@ lru_cache <- function(size = 10000) {
   function(key, value) {
     #value lazily forced if not found
     if (exists(key, lru)) {
-      hits <<- hits+1
+      hits <<- hits + 1L
       #move accessed value to front
       new_succ <- succ[[key]]
       new_pred <- pred[[key]]
@@ -80,7 +82,7 @@ lru_cache <- function(size = 10000) {
       succ$HEAD <<- key
       lru[[key]]
     } else {
-      misses <<- misses+1
+      misses <<- misses + 1L
       lru[[key]] <<- value
       #drop if entries exceeded
       while (used >= size) {
@@ -90,14 +92,14 @@ lru_cache <- function(size = 10000) {
         rm(list=last, envir=lru)
         rm(list=last, envir=pred)
         rm(list=last, envir=succ)
-        expired <<- expired + 1
-        used <<- used - 1
+        expired <<- expired + 1L
+        used <<- used - 1L
       }
       succ[[key]] <<- succ$HEAD
       pred[[succ$HEAD]] <<- key
       succ$HEAD <<- key
       pred[[key]] <<- "HEAD"
-      used <<- used + 1
+      used <<- used + 1L
       value
     }
   }
